#include "asset.h"
#include "arender.h"
#include "audiodevice.h"
#include "file.h"
#include "mainwindow.h"
#include <sys/resource.h>
#include <unistd.h>

ARender::ARender(MainWindow *mwindow)
{
	this->mwindow = mwindow;
	reset_parameters();
}

ARender::~ARender()
{
}

int ARender::reset_parameters()
{
	audio = 0;
	buffer = 0;
	return 0;
}

int ARender::arm_playback()
{
	int result = 0;
	synchronous = 1;
	thread_done = 0;
	audio = new AudioDevice(mwindow->asset);
	audio->set_software_sync(mwindow->software_sync);
	buffer_size = audio->start_playback();

	if(!result && buffer_size)
	{
		startup_lock.lock();
		mwindow->audio_file->set_audio_stream(mwindow->audio_stream);
		mwindow->audio_file->set_position(mwindow->current_percentage);
		bytes = mwindow->asset->bits / 8 * mwindow->asset->channels * buffer_size;
		buffer = new char[bytes];

		return 0;
	}
	else
	{
		delete audio;
		reset_parameters();
		return 1;
	}
	return 0;
}

int ARender::stop_playback()
{
	thread_done = 1;
	if(audio) audio->interrupt_playback();
	return 0;
}

int ARender::close_playback()
{
	audio->stop_playback();
	delete buffer;
	delete audio;
	reset_parameters();
	return 0;
}

int ARender::wait_for_startup()
{
	startup_lock.lock();
	startup_lock.unlock();
	return 0;
}

int ARender::wait_for_completion()
{
	join();
	return 0;
}

int ARender::renice()
{
	if(setpriority(PRIO_PROCESS, getpid(), mwindow->audio_priority) < 0) 
	{
		perror("ARender::renice");
		return 1;
	}
	return 0;
}


void ARender::run()
{
	renice();
	startup_lock.unlock();
	while(!thread_done && buffer_size)
	{
		buffer_length = buffer_size;

		mwindow->audio_file->lock_read();
		mwindow->audio_file->read_audio(buffer, buffer_length);
		mwindow->audio_file->unlock_read();
		if(!thread_done) audio->write_audio(buffer, buffer_length);
		if(!thread_done) thread_done = mwindow->audio_file->end_of_audio();
	}

// Flush the audio device
	if(audio->playing_back) audio->write_audio(0, 0);
	close_playback();
}

